package net.sf.itcb.module.blank.business.dao.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.stereotype.Repository;

import net.sf.itcb.module.blank.business.dao.BlankDao;
import net.sf.itcb.module.blank.schemas.types.BlankBean;

/**
 * DAO example that can be used
 * DAO must follow this implementation :
 * <ul>
 * <li>no exception management</li>
 * <li>no commit</li>
 * <li>no rollback</li>
 * <li>only data management and a connection release in a finally block</li>
 * <
 * </ul>
 * 
 * commit, rollback are dealing by the transaction manager.
 * 
 * @author Pierre Le Roux
 */
@Repository
public class BlankDaoImpl extends JdbcDaoSupport implements BlankDao{

	private static final String SQL_GETBYID =
            "select id, name from blank_table where id=?;";

	private static final String SQL_CREATE =
        	"insert into blank_table (name) values (?)";

	private static final String SQL_UPDATE =
    	"update blank_table set name=? where id=?";
	
	private static final String SQL_GETALL =
    	"select id, name from blank_table";
	
    public final BlankBean getBlankById(final Long id) throws Exception {

    	Connection con = null;
    	BlankBean blankBean = null;
		
		try {
			con = getConnection();

			PreparedStatement stmt = con.prepareStatement(SQL_GETBYID);
			
			stmt.setLong(1, id);
			stmt.execute();
			
			ResultSet rs = stmt.getResultSet();
			if(rs.next()) {
				blankBean = new BlankBean();
				blankBean.setId(rs.getLong(1));
				blankBean.setName(rs.getString(2));
			}			
					
		} finally {
			releaseConnection(con);
		}
		
		return blankBean;

    }

	@Override
	public BlankBean createBlank(String name) throws Exception {
		Connection con = null;
		BlankBean blankBean = null;
		try {
			con = getConnection();
			PreparedStatement stmt = con.prepareStatement(SQL_CREATE, Statement.RETURN_GENERATED_KEYS);
			
			stmt.setString(1, name);
			stmt.executeUpdate();
			
			ResultSet rs = stmt.getGeneratedKeys();
			if(rs.next()) {
				blankBean = new BlankBean();
				blankBean.setId(rs.getLong(1));
				blankBean.setName(name);
			}

		} finally {
			releaseConnection(con);
		}
		
		return blankBean;
	}

	@Override
	public void updateBlank(BlankBean blankBean) throws Exception {
		Connection con = null;
		try {
			con = getConnection();
			PreparedStatement stmt = con.prepareStatement(SQL_UPDATE, Statement.RETURN_GENERATED_KEYS);
			
			stmt.setString(1, blankBean.getName());
			stmt.setLong(2, blankBean.getId());
			stmt.executeUpdate();			

		} finally {
			releaseConnection(con);
		}
		
	}

	@Override
	public List<BlankBean> getAllBlankBean() throws Exception {
		Connection con = null;
    	List<BlankBean> listBlankBean = new ArrayList<BlankBean>();
		
		try {
			con = getConnection();

			PreparedStatement stmt = con.prepareStatement(SQL_GETALL);
			
			stmt.execute();
			
			ResultSet rs = stmt.getResultSet();
			BlankBean blankBean;
			while(rs.next()) {
				blankBean = new BlankBean();
				blankBean.setId(rs.getLong(1));
				blankBean.setName(rs.getString(2));
				listBlankBean.add(blankBean);
			}			
			
		} finally {
			releaseConnection(con);
		}
		
		return listBlankBean;
	}

}
