package net.sf.itcb.module.blank.business.service.impl;

import javax.annotation.Resource;

import net.sf.itcb.module.blank.business.dao.BlankDao;
import net.sf.itcb.module.blank.business.exception.BlankException;
import net.sf.itcb.module.blank.business.exception.BlankExceptionMappingErrors;
import net.sf.itcb.module.blank.business.service.BlankService;
import net.sf.itcb.module.blank.schemas.types.BlankBean;
import net.sf.itcb.module.blank.schemas.GetAllBlanksRequest;
import net.sf.itcb.module.blank.schemas.GetAllBlanksResponse;
import net.sf.itcb.module.blank.schemas.GetBlankRequest;
import net.sf.itcb.module.blank.schemas.GetBlankResponse;
import net.sf.itcb.module.blank.schemas.SaveBlankRequest;
import net.sf.itcb.module.blank.schemas.SaveBlankResponse;
import net.sf.itcb.module.blank.schemas.UpdateManyBlanksRequest;
import net.sf.itcb.module.blank.schemas.UpdateManyBlanksResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Configurable;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;

/**
 * Service implementation example.<br/>
 * Resources and dao are injected by Spring annotations.<br/>
 * No technical exception management (thrown by dafault to the upper level).<br/>
 * Only functional exception management.<br/>
 * No commit / No rollback. Transation manager does it by spring declaration.<br/>
 * 
 * The service implementation extends the service BlankService (inside blank-business module).
 * 
 * @author Pierre Le Roux
 */
@Service
@Configurable(preConstruction = true)
public class BlankServiceImpl implements BlankService {

	@Resource(name="blank-business-implResources")
	protected org.springframework.context.MessageSource messageSource;
	
	@Resource(name="blankDao")
	BlankDao blankDao;
	
	protected final Logger log = LoggerFactory.getLogger(getClass());
	
	@Override
	public GetBlankResponse getBlankById(GetBlankRequest getBlankRequest) throws Exception {
		GetBlankResponse getBlankResponse = new GetBlankResponse();
		getBlankResponse.setBlank(blankDao.getBlankById(getBlankRequest.getId()));
		return getBlankResponse;
	}
	
	@Override
	public SaveBlankResponse saveBlank(SaveBlankRequest saveBlankRequest) throws Exception  {
		BlankBean blankBean = saveBlankRequest.getBlank();
		SaveBlankResponse saveBlankResponse = new SaveBlankResponse();
		if(saveBlankRequest.getBlank().getId() == 0) {
			blankBean = blankDao.createBlank(blankBean.getName());
		}
		else {
			blankDao.updateBlank(blankBean);
		}
		saveBlankResponse.setBlank(blankBean);
		return saveBlankResponse;
	}
	
	@Override
	public UpdateManyBlanksResponse updateManyBlanks(UpdateManyBlanksRequest updateManyBlanksRequest) throws Exception {
		
		for (BlankBean currentBlankBean : updateManyBlanksRequest.getBlank()) {
			if(currentBlankBean.getId() == 0) {
				throw new BlankException(BlankExceptionMappingErrors.BLANK_NOT_SET_ID, messageSource.getMessage("blank-business-impl.exception.not_set_id", null, LocaleContextHolder.getLocale()));
			}
			blankDao.updateBlank(currentBlankBean);
		}
		return new UpdateManyBlanksResponse();
	}

	@Override
	public GetAllBlanksResponse getAllBlanks(GetAllBlanksRequest getAllBlanksRequest) throws Exception {
		GetAllBlanksResponse getAllBlanksResponse = new GetAllBlanksResponse();
		getAllBlanksResponse.getBlank().addAll(blankDao.getAllBlankBean());
		return getAllBlanksResponse;
	}
	
}
